<?php

/**
 *
 * Copyright © 2016 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace FiloBlu\OrderAdditionalData\Model\Plugin;

use Exception;
use FiloBlu\OrderAdditionalData\Model\OrderDataFactory;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;

use function serialize;

/**
 * Class OrderSave
 * Save Additional Data when saving Orders by OrderRepository
 * @package FiloBlu\OrderAdditionalData\Model\Plugin
 */
class OrderSave
{
    /** @var OrderDataFactory */
    protected $orderDataFactory;

    /**
     * OrderSave constructor.
     * @param OrderDataFactory $orderDataFactory
     */
    public function __construct(OrderDataFactory $orderDataFactory)
    {
        $this->orderDataFactory = $orderDataFactory;
    }

    /**
     * Save additional data on OAD data structures
     *
     * @param OrderRepositoryInterface $subject
     * @param OrderInterface $resultOrder
     * @return OrderInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @throws CouldNotSaveException
     */
    public function afterSave(
        OrderRepositoryInterface $subject,
        OrderInterface $resultOrder
    ) {
        /** @var OrderInterface $resultOrder */
        return $this->saveOrderAdditionalData($resultOrder);
    }

    /**
     * Save additional data for order
     *
     * @param OrderInterface $order
     * @return OrderInterface
     * @throws CouldNotSaveException
     */
    protected function saveOrderAdditionalData(OrderInterface $order)
    {
        $extensionAttributes = $order->getExtensionAttributes();

        if (null !== $extensionAttributes && null !== $extensionAttributes->getFbOad()) {
            $additionalDataObject = $extensionAttributes->getFbOad();
            $additionalData = [];
            foreach ($additionalDataObject->getCustomAttributes() as $customAttribute) {
                $additionalData[$customAttribute->getAttributeCode()] = $customAttribute->getValue();
            }
            try {
                $orderData = $this->orderDataFactory->create()->load($order->getEntityId(), 'order_id');
                $orderData->setOrderId($order->getEntityId());
                $orderData->setAdditionalData($this->serialize($additionalData));
                $orderData->save();
            } catch (Exception $e) {
                throw new CouldNotSaveException(
                    __('Could not save Additional Data for order: "%1"', $e->getMessage()),
                    $e
                );
            }
        }
        return $order;
    }

    /**
     * @param $data
     * @return string
     */
    public function serialize($data)
    {
        if (interface_exists(SerializerInterface::class) || class_exists(SerializerInterface::class)) {
            return ObjectManager::getInstance()->create(SerializerInterface::class)
                ->serialize($data);
        }

        return serialize($data);
    }
}
